==> fxt-2012.04.21.tbz: include/fxt/comb/catalan-gslex.h <==
#if !defined  HAVE_CATALAN_GSLEX_H__
#define       HAVE_CATALAN_GSLEX_H__
// This file is part of the FXT library.
// Copyright (C) 2012 Joerg Arndt
// License: GNU General Public License version 3 or later,
// see the file COPYING.txt in the main directory.


#include "fxttypes.h"


// Cf. comb/catalan-subset-lex.h for Catalan RGS in subset-lex order.


class catalan_gslex
// Catalan restricted growth strings (RGS),
// ordering similar to gslex (and subset-lex) order.
{
public:
    ulong *as_; // digits of the RGS: as_[k] <= as[k-1] + 1
    ulong tr_;  // current track
    ulong n_;   // Number of digits (paren pairs)
    char *str_; // paren string

public:
    catalan_gslex(ulong n)
    {
        n_ = n;

        as_ = new ulong[n_+ 1 + (n_==0)];  // incl. sentinel
//        as_[0] = +1; // sentinel !=0  to catch scan to the left
//        ++as_;  // nota bene
        if ( n_==0 )  as_[0] = 0;

        str_ = new char[2*n_+1];  str_[2*n_] = 0;
        first();
    }

    ~catalan_gslex()
    {
//        --as_;
        delete [] as_;
        delete [] str_;
    }

    void first()
    {
        for (ulong k=0; k<n_; ++k)  as_[k] = 0;
        tr_ = n_ - 1;
        // make things work for n <= 1:
        if ( n_ <= 1 )  tr_ = 0;
        else            as_[tr_] = 1;
    }

    bool next()
    // Loopless algorithm
    {
        ulong j = tr_;
        if ( j==0 )  return false;  // current is last

        const ulong aj = as_[j];
        if ( aj <= as_[j-1] )   // easy case 1: can increment
        {
            as_[j] = aj + 1;
            ++j;
            if ( j < n_ )
            {
                tr_ = n_ - 1;
                as_[tr_] = 1;
            }
            return true;
        }

        if ( aj != 1 )  // easy case 2: can detach
        {
            as_[j] = 0;
            tr_ = j - 1;
            return true;
        }

        // we have as[j]==1 and as[j-1]==0
        as_[j] = 0;
        if ( tr_ == 1 )  // created all-zero RGS (last)
        {
            tr_=0;
            return true;
        }
        // move the 1 to the left:
        as_[j-1] = 1;
        // put 1 at rightmost position:
        j = n_ - 1;
        as_[j] = 1;
        tr_ = j;
        return true;
    }

//    void last()
//    {
//    }
//
//    bool prev()
//    {
//    }

    const ulong *data()  const  { return as_; }

    const char* str()
    {
        for (ulong k=0; k<2*n_; ++k)  str_[k] = ')';
        for (ulong k=0,j=0; k<n_; ++k,j+=2)  str_[ j-as_[k] ] = '(';
        return str_;
    }
};
// -------------------------

#endif  // !defined HAVE_CATALAN_GSLEX_H__

==> fxt-2012.06.18.tbz: include/fxt/comb/catalan-gslex.h <==
#if !defined  HAVE_CATALAN_GSLEX_H__
#define       HAVE_CATALAN_GSLEX_H__
// This file is part of the FXT library.
// Copyright (C) 2012 Joerg Arndt
// License: GNU General Public License version 3 or later,
// see the file COPYING.txt in the main directory.


#include "fxttypes.h"


// Cf. comb/catalan-subset-lex.h for Catalan RGS in subset-lex order.


class catalan_gslex
// Catalan restricted growth strings (RGS),
// ordering similar to gslex (and subset-lex) order.
{
public:
    ulong *as_;  // digits of the RGS: as_[k] <= as[k-1] + 1
    ulong tr_;  // current track
    ulong n_;   // Number of digits (paren pairs)
    char *str_;  // paren string

public:
    explicit catalan_gslex(ulong n)
    {
        n_ = n;

        as_ = new ulong[n_+ 1 + (n_==0)];  // incl. sentinel
//        as_[0] = +1;  // sentinel !=0  to catch scan to the left
//        ++as_;  // nota bene
        if ( n_==0 )  as_[0] = 0;

        str_ = new char[2*n_+1];
        str_[2*n_] = 0;
        first();
    }

    ~catalan_gslex()
    {
//        --as_;
        delete [] as_;
        delete [] str_;
    }

    void first()
    {
        for (ulong k=0; k<n_; ++k)  as_[k] = 0;
        tr_ = n_ - 1;
        // make things work for n <= 1:
        if ( n_ <= 1 )  tr_ = 0;
        else            as_[tr_] = 1;
    }

    bool next()
    // Loopless algorithm
    {
        ulong j = tr_;
        if ( j==0 )  return false;  // current is last

        const ulong aj = as_[j];
        if ( aj <= as_[j-1] )   // easy case 1: can increment
        {
            as_[j] = aj + 1;
            ++j;
            if ( j < n_ )
            {
                tr_ = n_ - 1;
                as_[tr_] = 1;
            }
            return true;
        }

        if ( aj != 1 )  // easy case 2: can detach
        {
            as_[j] = 0;
            tr_ = j - 1;
            return true;
        }

        // we have as[j]==1 and as[j-1]==0
        as_[j] = 0;
        if ( tr_ == 1 )  // created all-zero RGS (last)
        {
            tr_=0;
            return true;
        }
        // move the 1 to the left:
        as_[j-1] = 1;
        // put 1 at rightmost position:
        j = n_ - 1;
        as_[j] = 1;
        tr_ = j;
        return true;
    }

//    void last()
//    {
//    }
//
//    bool prev()
//    {
//    }

    const ulong *data()  const  { return as_; }

    const char* str()
    {
        for (ulong k=0; k<2*n_; ++k)  str_[k] = ')';
        for (ulong k=0, j=0;  k<n_;  ++k, j+=2)  str_[ j-as_[k] ] = '(';
        return str_;
    }
};
// -------------------------

#endif  // !defined HAVE_CATALAN_GSLEX_H__
